function Q = trim(P,range)

% TRIM -- adjust the length of matrix polynomial
% 
%        Q = trim(P,[kmin,kmax])
%
% The main uses of this routine are:
%
%   1. stripping away zero matrices at the ends
%   2. selecting a sub-polynomial
%   3. insuring a fixed range of exponents
%
% This routine first sets all matrix entries below tolerance to
% zero. Tolerance is the value of global variable MPOLY_TOLERANCE,
% or 1.e-12 if that variable is not set. If you set MPOLY_TOLERANCE
% to 0, no entries will be touched.
%
% Then, the range of exponents is adjusted either to [kmin,kmax]
% (if given), or to the shortest possible range by trimming zeros
% coefficients from the ends. 
%
% If the global variable MPOLY_NOTRIM is set to a nonzero value,
% zero matrices at the ends will not be stripped.

% Copyright (c) 2004 by Fritz Keinert (keinert@iastate.edu),
% Dept. of Mathematics, Iowa State University, Ames, IA 50011.
% This software may be freely used and distributed for non-commercial
% purposes, provided this copyright statement is preserved, and
% appropriate credit for its use is given.
%
% Last update: Feb 20, 2004

global MPOLY_TOLERANCE MPOLY_NOTRIM

if isempty(MPOLY_TOLERANCE)
    tol = 1.e-12;
else
    tol = MPOLY_TOLERANCE;
end

if (nargin < 2)
    range = [];
end

% if the 'range' argument is missing and MPOLY_NOTRIM is set,
% do nothing

if ~isempty(MPOLY_NOTRIM)
    if (MPOLY_NOTRIM & isempty(range))
	Q = P;
	return;
    end
end

% if P is symbolic, try to simplify it
% Don't call @mpoly/simplify! That causes an infinite loop
if ~isnumeric(P)
    n = size(P.coef);
    P.coef = reshape(simplify(P.coef(:)),n);
end

% If P is numeric, zero all entries below tolerance.
% Symbolic entries are assumed to be 100% correct,
% so we don't round

C = P.coef;
[n1,n2,n3] = size(C);
kmin = P.min;
kmax = get(P,'max');

if isnumeric(C)
    C(abs(C) < tol) = 0;
end

if isempty(range)
% trim zeros from the ends

    if iszero(C)
	% all coefficients are 0; return a single zero matrix with subscript 0
	if (isa(C,'sym'))
	    C = sym(zeros(n1,n2));
	else
	    C = zeros(n1,n2);
	end
	kmin = 0;
    else
	% strip zeros from front
	while iszero(C(:,:,1))
	    kmin = kmin + 1;
	    C = C(:,:,2:end);
	end
	% strip zeros from end
	while iszero(C(:,:,end))
	    C = C(:,:,1:end-1);
        end
    end
else        
% adjust to given range

    if (range(1) > range(2))
	error('empty range');
    end
    
    if (range(1) < kmin)
	% pad with initial zeros
	k = kmin-range(1);
	C = cat(3,zeros(n1,n2,k),C);
	kmin = kmin - k;
    elseif (range(1) > kmin)
	% strip some matrices off the front
	k = range(1) - kmin;
	C = C(:,:,k+1:end);
	kmin = range(1);
    end
    cmax = kmin+size(C,3)-1;
    if (range(2) > cmax)
	% add extra zeros at end
	k = range(2) - cmax;
	C = cat(3,C,zeros(n1,n2,k));
    elseif (range(2) < cmax)
	% strip some matrices off the end
	k = cmax - range(2);
	C = C(:,:,1:end-k);
    end
end
Q = mpoly(C,kmin,P.type,P.m,P.r);
